using System;
using System.Collections;

namespace pixtek
{
	public class CommandLineParsing
	{

		public class SwitchValue
		{
			public string Switch;
			public string Value;

			public SwitchValue(string _switch, string _value)
			{
				Switch=_switch;
				Value=_value;
			}
		}

		private ArrayList	m_switchValue;
		private ArrayList	m_switchBool;
		private ArrayList	m_freeValues;

		public CommandLineParsing()
		{
		}

		public CommandLineParsing(string commandLine, string[] regSwitch)
		{
			DoParsing(commandLine, regSwitch);
		}

		public CommandLineParsing(string[] args, string[] regSwitch)
		{
			DoParsing(args, regSwitch);
		}

		private ArrayList ParseLine(string commandLine)
		{
			// Retrouver chaque lment sur la "pile" des arguments
			ArrayList result = new ArrayList();

			char[] line = commandLine.ToCharArray();
			int index=0;
			while(index<line.Length)
			{
				// Passer les espaces
				while (index<line.Length && line[index]==' ')
					index++;
				int i = index+1;
				if (index<line.Length)
				{
					switch(line[index])
					{
						case '"':
							// Trouver le double quote fermant
							while (i<line.Length && line[i]!='"')
								i++;
							result.Add(commandLine.Substring(index+1, i-index-1));
							break;
						case (char)39: // '
							// Trouver le simple quote fermant
							while (i<line.Length && line[i]!=39)
								i++;
							result.Add(commandLine.Substring(index+1, i-index-1));
							break;
						case '#': 
							// Trouver le simple quote fermant
							while (i<line.Length && line[i]!='#')
								i++;
							result.Add(commandLine.Substring(index+1, i-index-1));
							break;
						default:
							// Trouver l'espace fin de chane
							while (i<line.Length && line[i]!=' ')
								i++;
							result.Add(commandLine.Substring(index, i-index));
							break;
					}
				}
				index = i+1;
			}
			return result;
		}

		private ArrayList ParseMacro(ArrayList args)
		{
			ArrayList result = new ArrayList();
			foreach (string s in args)
			{
				if (s[0]=='@')
				{
					string fileName = s.Substring(1);
					using (System.IO.StreamReader sr = new System.IO.StreamReader(fileName))
					{
						result.AddRange(ParseMacro((ParseLine(sr.ReadToEnd()))));
					}
				}
				else
					result.Add(s);
			}
			return result;
		}

		public void DoParsing(ArrayList args, string[] regSwitch)
		{
			m_switchValue = new ArrayList();
			m_freeValues = new ArrayList();
			m_switchBool = new ArrayList();

			// Regarder si nous avons un @ qui indique un fichier de paramtre
			ArrayList args2 = ParseMacro(args);
			for (int i=0; i<args2.Count; i++)
			{
				string item = args2[i].ToString();
				string firstChar = item.Substring(0, 1);
				if (firstChar=="-" || firstChar=="/" || firstChar=="\\")
				{
					string temp = item.Substring(1).ToUpper();
					bool validSwitch=false;
					// Est-ce un switch connu ?
					foreach (string s in regSwitch)
					{
						if (s.ToUpper()==temp)
						{
							validSwitch=true;
							break;
						}
					}

					if (validSwitch)
					{
						m_switchValue.Add(new SwitchValue(temp, args2[i+1].ToString()));
						i++;
					}
					else
						m_switchBool.Add(temp);
				}
				else
					m_freeValues.Add(item);
			}
		}

		public void DoParsing(string[] args, string[] regSwitch)
		{
			ArrayList tmp = new ArrayList(args);
			DoParsing(tmp, regSwitch);
		}

		public void DoParsing(string commandLine, string[] regSwitch)
		{
			DoParsing(ParseLine(commandLine), regSwitch);
		}

		/// <summary>
		/// Get the string correspondig to the Argv
		/// </summary>
		/// <param name="Argv">use -option formation (provide the dash)</param>
		/// <param name="Default">Default value if not found</param>
		/// <param name="index">Index if several argument of the same text can be provided. Begin 0</param>
		/// <returns>The associated value</returns>
		public string GetSwitch(string Argv, string Default, int index)
		{
			Argv = Argv.ToUpper();
			string result = Default;
			foreach (SwitchValue sw in m_switchValue)
			{
				if (sw.Switch==Argv)
				{
					if (index==0)
					{
						result = sw.Value;
						break;
					}
					else
						index--;
				}
			}
			return result;
		}

		public string GetSwitch(string Argv, string Default)
		{
			return GetSwitch(Argv, Default, 0);
		}

		public string GetSwitch(string Argv, int index)
		{
			return GetSwitch(Argv, string.Empty, index);
		}

		public string GetSwitch(string Argv)
		{
			return GetSwitch(Argv, string.Empty, 0);
		}

		public bool GetBool(string Argv)
		{
			bool bResult=false;
			foreach (string s in m_switchBool)
			{
				if (s==Argv.ToUpper())
				{
					bResult=true;
					break;
				}
			}
			return bResult;
		}

		// Returns free text that is not linked to an option switch
		public string GetFreeText(int index)
		{
			string result=string.Empty;
			if (index<m_freeValues.Count)
			{
				foreach (string s in m_freeValues)
				{
					if (index==0)
					{
						result = s;
						break;
					}
					index--;
				}
			}
			return result;
		}
	}
}
