using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace url2jpeg
{
    public partial class Main : Form
    {
        lang m_lang;
        langCountry m_currentCountry;
        userSettings m_userSettings;
        delegate void AddTab(bool focus);

        public Main()
        {
            m_userSettings = userSettings.LoadFromFile(GetUserXmlFileName());
            userSettings.SetInstance(m_userSettings);

            InitializeComponent();
        }

        void RemoveAllTabExcept(TabPage page)
        {
            // Delete all other tabs
            List<Control> toDelete = new List<Control>();
            foreach (Control ctrl in tabControl1.Controls)
            {
                if (ctrl != page)
                    toDelete.Add(ctrl);
            }

            foreach (Control ctrl in toDelete)
                tabControl1.Controls.Remove(ctrl);
        }

        void AddNewVersionTab(bool bFocus)
        {
            tabControl1.Controls.Add(tabNewVersion);
            if (bFocus)
                RemoveAllTabExcept(tabNewVersion);
        }

        void CheckVersion(IAsyncResult result)
        {
            try
            {
                System.Net.IPAddress[] availVersion = System.Net.Dns.EndGetHostAddresses(result);
                if (availVersion.Length == 1)
                {
                    string[] items = availVersion[0].ToString().Split('.');
                    if (items.Length == 4)
                    {
                        // version.url2jpeg.com  v1.v2.f1.f2
                        // v1 et v2 = version majeur, mineur
                        // f1*256+f2 = build obligatoire pour fonctionner
                        bool bNewVersion = false, bMajor = false;
                        if (Convert.ToInt32(items[0]) > 1)
                            bNewVersion = true;
                        else if (Convert.ToInt32(items[1]) > 1)
                            bNewVersion = true;

                        int force = Convert.ToInt32(items[2]) * 256 + Convert.ToInt32(items[3]);
                        /*
                        ////
                        force = 1;
                        bNewVersion = true;
                        ////
                        */
                        // Ici le 0 = numro de build interne, changer cette valeur pour nouvelle version essentielle
                        if (bNewVersion && force > 0) // cela laisse 65536 possibilits de forcer une maj
                            bMajor = true;

                        if (bNewVersion)
                            tabControl1.Invoke(new AddTab(AddNewVersionTab), bMajor);
                    }
                }
            }
            catch { }
        }

        public string GetLangXmlFileName()
        {
            //return System.IO.Path.GetDirectoryName(Application.ExecutablePath) + "\\lang.xml";
            return Tag.ToString() + "\\lang.xml";
        }

        public string GetUserXmlFileName()
        {
            return System.Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments)+"\\url2jpeg.xml";
        }

        private void Main_Load(object sender, EventArgs e)
        {
            // Remove new version tab
            tabControl1.Controls.Remove(tabNewVersion);

            pixtek.CommandLineParsing clp = new pixtek.CommandLineParsing(Environment.GetCommandLineArgs(), new string[] { "script", "destination", "language" });

            string script = clp.GetSwitch("script");
            string destination = clp.GetSwitch("destination");
            string language = clp.GetSwitch("language");
            bool bExecute = clp.GetBool("run");
            bool bMinimized = clp.GetBool("minimized");
            bool bExit = clp.GetBool("exit");

            m_lang = lang.LoadXML(GetLangXmlFileName());
            if (string.IsNullOrEmpty(m_userSettings.m_defaultLang) && !string.IsNullOrEmpty(language))
                m_userSettings.m_defaultLang = language;

            if (string.IsNullOrEmpty(m_userSettings.m_defaultLang))
                m_userSettings.m_defaultLang = "English";

            m_currentCountry = m_lang.FindByCountry(m_userSettings.m_defaultLang);
            if (m_currentCountry == null)
            {
                m_currentCountry = new langCountry(m_userSettings.m_defaultLang);
                m_lang.m_locale.Add(m_currentCountry);
            }

            settings1.SetLang(m_lang, m_userSettings.m_defaultLang);

            if (bMinimized)
                WindowState = FormWindowState.Minimized;

            if (script.Length > 0)
            {
                script1.LoadScript(script);
                if (bExecute && destination.Length > 0)
                {
                    RemoveAllTabExcept(tabScripting);
                    script1.Run(false, destination);
                    if (bExit)
                        Close();
                }
            }
            else
            {
                // Start the version verification thread
                System.Net.Dns.BeginGetHostAddresses("version.url2jpeg.com", CheckVersion, null);
            }

            // For beta test only
            while (!settings.KeyCorrect(m_userSettings.m_email, Convert.ToInt64(m_userSettings.m_regkey)))
            {
                licence lic = new licence();
                if (lic.ShowDialog() == DialogResult.Cancel)
                {
                    Close();
                    break;
                }
            }


        }

        private void DoTranlate()
        {
            navigation1.DoTranslate(m_currentCountry);
            capture1.DoTranslate(m_currentCountry);
            script1.DoTranslate(m_currentCountry);
            settings1.DoTranslate(m_currentCountry);
            foreach (Control ctrl in tabControl1.Controls)
            {
                ctrl.Text = m_currentCountry.GetItem(ctrl.Name, ctrl.Text);
            }
        }

        private void settings1_LanguageChangedEvent(string newLang)
        {
            m_currentCountry = m_lang.FindByCountry(newLang);
            if (m_currentCountry != null)
            {
                m_userSettings.m_defaultLang = newLang;
                DoTranlate();
            }
        }

        private void Main_FormClosing(object sender, FormClosingEventArgs e)
        {
            script1.Exit();
            try
            {
                m_userSettings.SaveToFile(GetUserXmlFileName());
                m_lang.SaveXML(GetLangXmlFileName());
            }
            catch { }
        }

        private void navigation1_GoToCaptureEvent(url2jpegCtn.navigator navigator)
        {
            Cursor.Current = Cursors.WaitCursor;
            tabControl1.SelectTab(tabCapture);
            capture1.SetNavigator(navigator);
            capture1.RefreshDisplay();
            Cursor.Current = Cursors.Arrow;
        }

        private void capture1_GenScriptEvent()
        {
            scriptData data = new scriptData();
            if (navigation1.FillForScript(data))
            {
                capture1.FillForScript(data);
                script1.NewScript(data);
                tabControl1.SelectTab(tabScripting);
            }
        }

        private void tabControl1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (tabControl1.SelectedTab == tabNewVersion)
            {
                newVersion1.Navigate();
            }
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start(((Control)sender).Text);
        }
    }
}