// pixtek.AutoScale
// (c) Pixel Technology, August 2001
// info@pixel-technology.com

// Gestion d'un ratio. Amlioration possible avec le clipping.

using System;
using System.Drawing;
using System.Windows.Forms;

namespace pixtek
{
	/// <summary>
	/// This class do all the job for placing and moving
	/// a rectangle whith mouse
	/// </summary>
	public class PlaceObject 
	{

		/// <summary>
		/// Personalize this class and assign it to PlaceObject if you need
		/// scaling functionnality
		/// </summary>
		public class Scaling
		{
			static public int MulDiv(int n1, int n2, int d)
			{
				double i,j,k;
				i=n1;
				j=n2*i;
				k=d;
				i = j/d;
				return Convert.ToInt32(i);
			}
			/// <summary>
			/// Overridable, point transformation, use to draw
			/// </summary>
			/// <param name="point">point in world reference</param>
			/// <returns>translated point to device reference</returns>
			public virtual Point PointWorldToDevice(Point point)
			{
				return point;
			}

			/// <summary>
			/// Overridable, point transformation, use for cursor
			/// </summary>
			/// <param name="point">point in device reference</param>
			/// <returns>translated point to world reference</returns>
			public virtual Point PointDeviceToWorld(Point point)
			{
				return point;
			}

			/// <summary>
			/// Overridable, size transformation, use to draw
			/// </summary>
			/// <param name="size">size in world reference</param>
			/// <returns>size in device reference</returns>
			public virtual Size SizeWorldToDevice(Size size)
			{
				return size;
			}

			/// <summary>
			/// Overridable, size transformation, use for cursor
			/// </summary>
			/// <param name="size">size in world reference</param>
			/// <returns>size in device reference</returns>
			public virtual Size SizeDeviceToWorld(Size size)
			{
				return size;
			}
		}

		private enum State { NoSelection, Moving };

		public const int ResizeLeft			= 17;
		public const int ResizeRight		= 20;
		public const int ResizeTop			= 10;
		public const int ResizeBottom		= 34;
		public const int Move				= 18;
		public const int ResizeLeftTop		= 9;
		public const int ResizeRightTop		= 12;
		public const int ResizeLeftBottom	= 33;
		public const int ResizeRightBottom	= 36;

		/// <summary>
		/// Subscribe this event to reflect cursor changement
		/// </summary>
		public delegate void SetCursorHandler (PlaceObject sender, Cursor cursor, int areaIndex);
		public event SetCursorHandler SetCursorEvent;

		/// <summary>
		/// Subscribe this event for use drag feedback
		/// </summary>
		public delegate void DragPositionHandler (PlaceObject sender);
		public event DragPositionHandler DragPositionEvent;

		/// <summary>
		/// Subscribe this event for final position feedback
		/// </summary>
		public delegate void FinalPositionHandler (PlaceObject sender);
		public event FinalPositionHandler FinalPositionEvent;

		/// <summary>
		/// Subscribe this event to know when mouse is outside objet because of clipping rectangle
		/// </summary>
		public delegate void MouseOutsideHandler (PlaceObject sender, bool bOutside);
		public event MouseOutsideHandler MouseOutisedeEvent;

		/// <summary>
		/// Subscribe this event to knwo when element is selected or deselected
		/// </summary>
		public delegate void SelectionHandler (PlaceObject sender, bool bSelected);
		public event SelectionHandler SelectionEvent;

		private		int			resizeDirection=0x1FF;// all directions
		protected	Rectangle	clippingRectangle;
		protected	Rectangle	initialRectangle;
		protected	Rectangle	currentRectangle;
		protected	double		ratio = 0.0;
		public		int			minWidth=0;
		public		int			minHeight=0;
		private		int			gripWidth=6;
		private		int			gripHeight=6;
		private		State		state;
		private		int			startIndex;
		private		int			startDx;
		private		int			startDy;
		private		int			startX;
		private		int			startY;
		private		bool		bMouseOutside;
		private		int			iMouseOutside;
		private		Scaling		scale;
		private		Cursor[]	cursor = {
										 Cursors.SizeNWSE,
										 Cursors.SizeNS,
										 Cursors.SizeNESW,
										 Cursors.SizeWE,
										 Cursors.SizeAll,
										 Cursors.SizeWE,
										 Cursors.SizeNESW,
										 Cursors.SizeNS,
										 Cursors.SizeNWSE};

		private struct SensitiveRectangle
		{
			public int			Mask;
			public Rectangle	Area;
			public Point		Origin;
		}
		private bool		bSelected = false;
		private	bool		bFullDrag = false;			// Allow full drag ?
		public bool			paint=true;					// Is initial position painted ?
		public bool			paintSelection=true;		// Is selected item painted ?
		public bool			paintGrip=true;				// Does grip painted ?
		public Pen			normalPen=null;				// Pen use for initial position rectangle
		public Pen			selectionPen=null;			// Pen use for selected state painting
		public Pen			gripPen=null;				// Pen for grip
		public Brush		gripBrush=null;				// Brush for grip


		/// <summary>
		/// Rectangle index are the following
		/// 0  1  2
		/// 3  4  5
		/// 6  7  8
		/// </summary>
		private SensitiveRectangle[] area = new SensitiveRectangle[9];

		/// <summary>
		/// Default constructor
		/// </summary>								 
		public PlaceObject()
		{
			state=State.NoSelection;
			scale=new Scaling();
		}

		/// <summary>
		/// Advanced constructor
		/// </summary>								 
		public PlaceObject(Scaling Scale)
		{
			state=State.NoSelection;
			scale=Scale;
		}
		
		/// <summary>
		/// Assign scaling to PlaceObject
		/// </summary>
		public Scaling Scale
		{
			get { return scale; }
			set { scale=value; }
		}

		/// <summary>
		/// Specifies a value indicating whether the item is selected.
		/// </summary>
		public bool Selected
		{
			get { return bSelected; }
			set { bSelected = value; OnSelected(bSelected); }
		}

		/// <summary>
		/// Sensitive direction available
		/// </summary>
		public int ResizeDirection
		{
			set { resizeDirection=value; }
			get { return resizeDirection;}
		}

		/// <summary>
		/// The clipping rectangle where selection is possible
		/// </summary>
		public Rectangle ClippingRectangle
		{
			set { clippingRectangle=value;}
			get { return clippingRectangle;}
		}

		/// <summary>
		/// The initial rectangle
		/// </summary>
		public Rectangle InitialRectangle
		{
			set 
			{ 
				initialRectangle=value; 
				currentRectangle=value;
				BuildRectangleIndex(ref initialRectangle, ref area);
			}
			get { return initialRectangle;}
		}

		/// <summary>
		/// The current rectangle moved by user
		/// </summary>
		public Rectangle CurrentRectangle
		{
			set { currentRectangle = value; }
			get { return currentRectangle;}
		}

		/// <summary>
		/// Assign a ratio to respect to
		/// </summary>
		public double Ratio
		{
			get { return ratio; }
			set 
			{ 
				ratio = value;	
				InitialRectangle = CheckRatio(InitialRectangle,0);
				CurrentRectangle = InitialRectangle;
			}
		}

		/// <summary>
		/// Size in pixel for the sensitive rectangles
		/// </summary>
		public int GripSize
		{
			set { gripWidth = gripHeight = value; BuildRectangleIndex(ref initialRectangle, ref area); }
		}

		/// <summary>
		/// Width in pixel for the sensitive rectangles
		/// </summary>
		public int GripWidth
		{
			set { gripWidth = value; BuildRectangleIndex(ref initialRectangle, ref area); }
			get { return gripWidth;}
		}

		/// <summary>
		/// Height in pixel for the sensitive rectangles
		/// </summary>
		public int GripHeight
		{
			set { gripHeight = value; BuildRectangleIndex(ref initialRectangle, ref area);}
			get { return gripHeight;}
		}

		/// <summary>
		/// Minimum width
		/// </summary>
		public int MinWidth
		{
			set { minWidth=value; }
			get { return minWidth;}
		}

		/// <summary>
		/// Minimun height
		/// </summary>
		public int MinHeight
		{
			set { minHeight=value;}
			get { return minHeight;}
		}

		public bool FullDrag
		{
			set { bFullDrag = value; }
			get { return bFullDrag; }
		}

		/// <summary>
		/// Fast SensitiveRectangle assignement
		/// </summary>
		private void SetRect (ref SensitiveRectangle srect, int ox, int oy, int x, int y, int w, int h, int mask)
		{
			srect.Origin = new Point(ox, oy);
			srect.Area = new Rectangle(x,y,w,h);
			srect.Mask=mask;
		}

		/// <summary>
		/// Build list of sensitive rectangle
		/// </summary>
		private void BuildRectangleIndex (ref Rectangle rect, ref SensitiveRectangle[] area)
		{
			int offsetx = GripWidth/2;
			int offsety = GripHeight/2;
			int lx = rect.Left-offsetx;
			int mx = lx+rect.Width/2;
			int rx = rect.Right-offsetx;
			int ty = rect.Top-offsety;
			int my = ty+rect.Height/2;
			int by = rect.Bottom-offsety;
			int left=rect.Left;
			int right=rect.Right;
			int top=rect.Top;
			int bottom=rect.Bottom;
			int middlex=left+rect.Width/2;
			int middley=top+rect.Height/2;

			if (area.Length>=9)
			{
				SetRect (ref area[0], left, top, lx, ty, GripWidth, GripHeight, ResizeLeftTop);
				SetRect (ref area[1], middlex, top, mx, ty, GripWidth, GripHeight, ResizeTop);
				SetRect (ref area[2], right, top, rx, ty, GripWidth, GripHeight, ResizeRightTop);
				SetRect (ref area[3], left, middley, lx, my, GripWidth, GripHeight, ResizeLeft);
				SetRect (ref area[4], middlex, middley, mx, my, GripWidth, GripHeight, Move);
				SetRect (ref area[5], right, middley, rx, my, GripWidth, GripHeight, ResizeRight);
				SetRect (ref area[6], left, bottom, lx, by, GripWidth, GripHeight, ResizeLeftBottom);
				SetRect (ref area[7], middlex, bottom, mx, by, GripWidth, GripHeight, ResizeBottom);
				SetRect (ref area[8], right, bottom, rx, by, GripWidth, GripHeight, ResizeRightBottom);
			}
		}

		/// <summary>
		/// Find if (x,y) is part of an active sensitive area
		/// </summary>
		private int GetAreaIndex(int x, int y)
		{
			for(int i = 0; i < 9; i++)
				if ((resizeDirection & area[i].Mask) == area[i].Mask && area[i].Area.Contains(x, y))
					return i;

			// Special full drag mode
			if (bFullDrag && (resizeDirection & area[4].Mask)==area[4].Mask && currentRectangle.Contains(x,y))
				return 4;

			return -1;
		}

		/// <summary>
		/// Do rectangle translation and resize
		/// </summary>
		/// <param name="x">x offset</param>
		/// <param name="y">y offset</param>
		/// <returns>new position that respect clipping rectangle</returns>
		private Rectangle MoveSelection(int x, int y)
		{
			int mask = area[startIndex].Mask;

			int dx = x-startX+startDx;
			int dy = y-startY+startDy;

			Rectangle workingRectangle = InitialRectangle;
			
			if (mask == Move)
			{
				workingRectangle.X = Math.Max(clippingRectangle.X,workingRectangle.X+x-startX);
				if (workingRectangle.Right >= clippingRectangle.Right)
					workingRectangle.X = clippingRectangle.Right - workingRectangle.Width;
				workingRectangle.Y = Math.Max(clippingRectangle.Y,workingRectangle.Y+y-startY);
				if (workingRectangle.Bottom >= clippingRectangle.Bottom)
					workingRectangle.Y = clippingRectangle.Bottom - workingRectangle.Height;
			} 
			else
			{
				if ((mask & 1) != 0) // Left
				{
					dx = Math.Min(dx,workingRectangle.Width-minWidth);
					workingRectangle.Width -= dx;
					workingRectangle.X += dx;
				}
				if ((mask & 4) != 0) // Right
				{
					dx = Math.Max(dx,-workingRectangle.Width+minWidth);
					workingRectangle.Width += dx;
				}
				if ((mask & 8) != 0) // Top
				{
					dy = Math.Min(dy,workingRectangle.Height-minHeight);
					workingRectangle.Height -= dy;
					workingRectangle.Y += dy;
				}
				if ((mask & 32) != 0) // Bottom
				{
					dy = Math.Max(dy,-workingRectangle.Height+minHeight);
					workingRectangle.Height += dy;
				}
		
				workingRectangle = CheckRatio(workingRectangle,mask);
				workingRectangle.Intersect(clippingRectangle);
			}

			
			return workingRectangle;
		}

		private Rectangle CheckRatio(Rectangle rect, int mask)
		{
			if (ratio == 0)
				return rect;

			Rectangle finalRect = rect;
			double localRatio = (double)rect.Width / rect.Height;

			if (mask == 0)
			{
				finalRect.Height = (int)(finalRect.Width / ratio);
			}
			else if (mask == ResizeLeft || mask == ResizeRight || mask == ResizeTop || mask == ResizeBottom)
			{
				if ((mask & 16) != 0)	finalRect.Height = (int)(finalRect.Width / ratio);
				if ((mask & 2) != 0)	finalRect.Width = (int)(finalRect.Height * ratio);
				if ((mask & 1) != 0)	finalRect.X = rect.Right - finalRect.Width;
				if ((mask & 8) != 0)	finalRect.Y = rect.Bottom - finalRect.Height;
			} 
			else
			{
				if (localRatio > ratio)		
				{
					finalRect.Height = (int)(finalRect.Width / ratio);
					if ((mask & 8) != 0)	finalRect.Y = rect.Bottom - finalRect.Height;
				}
				else						
				{
					finalRect.Width = (int)(finalRect.Height * ratio);
					if ((mask & 1) != 0)	finalRect.X = rect.Right - finalRect.Width;
				}
			}	
		
			return finalRect;
		}

		/// <summary>
		/// Draw small rectangle representing sensitive area
		/// </summary>
		/// <param name="dc">The graphics object</param>
		/// <param name="pen">The pen to use or null if DrawRectangle not needed</param>
		/// <param name="brush">The brush to use or null if FillRectangle not needed</param>
		public void DrawSensitiveArea (Graphics dc, Pen pen, Brush brush)
		{
			SensitiveRectangle[] localarea=null;

			switch (state)
			{
				case State.NoSelection:
					localarea=area;
					break;
				case State.Moving:
					localarea=new SensitiveRectangle[9];
					BuildRectangleIndex(ref currentRectangle, ref localarea);
					break;
			}

			for (int i=0; i<9 ;i++)
			{
				if ((resizeDirection & area[i].Mask) == area[i].Mask)
					GripOwnerDraw(dc, i, pen, brush, 
								  new Rectangle	(scale.PointWorldToDevice(localarea[i].Area.Location), scale.SizeWorldToDevice(localarea[i].Area.Size)));
			}
		}

		/// <summary>
		/// Override this to paint inside the selection
		/// </summary>
		/// <param name="absPosition"></param>
		public virtual void InsideDraw (Graphics dc, Rectangle absPosition)
		{
		}

		/// <summary>
		/// Override this to draw grip yourself
		/// </summary>
		/// <param name="dc">Graphics context</param>
		/// <param name="areaIndex">the actual grip to draw 0=top,left 9=bottom,right</param>
		/// <param name="pen">pen to use</param>
		/// <param name="brush">brush to use</param>
		/// <param name="drawingZone">clipping rectangle to draw into</param>
		public virtual void GripOwnerDraw (Graphics dc, int areaIndex, Pen pen, Brush brush, Rectangle drawingZone)
		{
			if (brush!=null)
				dc.FillRectangle(brush, drawingZone);
			if (pen!=null)
				dc.DrawRectangle(pen, drawingZone);
		}

		/// <summary>
		/// Provide painting support
		/// </summary>
		/// <param name="dc"></param>
		/// <param name="normalPen">could be null</param>
		/// <param name="selectionPen">could be null</param>
		/// <param name="gripPen">could be null</param>
		/// <param name="gripBrush">could be null</param>
		public void Paint(Graphics dc, Pen normalPen, Pen selectionPen, Pen gripPen, Brush gripBrush)
		{
			Pen pen;
			Brush brush;
			Region oldClip = dc.Clip;

			if (Selected)
			{
				if (paintSelection)
				{
					pen = (this.selectionPen == null) ? selectionPen : this.selectionPen;
					Rectangle deviceRect = RectWorldToDevice(currentRectangle);
					dc.Clip = new Region(deviceRect);
					InsideDraw(dc, deviceRect);
					dc.Clip = oldClip;
					dc.DrawRectangle(pen, deviceRect);
				}
				if (paintGrip)
				{
					pen = (this.gripPen == null) ? gripPen : this.gripPen;
					brush = (this.gripBrush == null) ? gripBrush : this.gripBrush;
					DrawSensitiveArea(dc, pen, brush);
				}
			} 
			else
			{
				if (paint)
				{
					pen = (this.normalPen == null) ? normalPen : this.normalPen;
					Rectangle deviceRect = RectWorldToDevice(initialRectangle);
					dc.Clip = new Region(deviceRect);
					InsideDraw(dc, deviceRect);
					dc.Clip = oldClip;
					dc.DrawRectangle(pen, deviceRect);
				}
			}
		}


		/// <summary>
		/// To know when mouse is over a sensitive area
		/// </summary>
		/// <param name="x"></param>
		/// <param name="y"></param>
		/// <returns>true if over a sensitive area</returns>
		public bool IsSensitivePoint (int x, int y)
		{
			bool bResult=false;

			Point point = scale.PointDeviceToWorld(new Point(x,y));
			if (GetAreaIndex(point.X, point.Y)!=-1)
				bResult=true;
			return bResult;
		}

		public bool IsInsideElement (int x, int y)
		{
			bool bResult=false;
			Point point = scale.PointDeviceToWorld(new Point(x,y));
			if ((GetAreaIndex(point.X, point.Y)!=-1) || initialRectangle.Contains(point.X, point.Y))
				bResult=true;
			return bResult;
		}

		/// <summary>
		/// Call this method when MouseMove event received
		/// </summary>
		/// <param name="x"></param>
		/// <param name="y"></param>
		public void MouseMove (int x, int y)
		{
			Point point = scale.PointDeviceToWorld(new Point(x,y));
			x=point.X;
			y=point.Y;

			int i;
			switch (state)
			{
				case State.NoSelection:
					i=GetAreaIndex(x,y);
					if (i!=-1)
						OnSetCursor(cursor[i],i);
					else
						OnSetCursor(Cursors.Default,-1);
					break;
				case State.Moving:
					currentRectangle=MoveSelection(x,y);
					OnDragPosition();
					Rectangle tmpRect = currentRectangle;
					tmpRect.Inflate (GripWidth, GripHeight);
					bool bOutside = !tmpRect.Contains(x,y);
					if (bMouseOutside!=bOutside)
					{
						bMouseOutside=bOutside;
						iMouseOutside=10;
					}
					else
					{
						if (iMouseOutside==1)
							OnMouseOutside(bMouseOutside,area[startIndex].Mask);
						if (iMouseOutside>0)
							iMouseOutside--;
					}

					break;
			}
		}

		/// <summary>
		/// Call this method when MouseDown event received
		/// </summary>
		/// <param name="x"></param>
		/// <param name="y"></param>
		public void MouseDown (int x, int y)
		{
			Point point = scale.PointDeviceToWorld(new Point(x,y));
			x=point.X;
			y=point.Y;

			int i;
			switch (state)
			{
				case State.NoSelection:
					i=GetAreaIndex(x,y);
					if (i!=-1)
					{
						state=State.Moving;
						startIndex=i;
						startDx=x-area[i].Origin.X;
						startDy=y-area[i].Origin.Y;
						startX=x;
						startY=y;
						currentRectangle=MoveSelection(x,y);
						OnDragPosition();
						bMouseOutside=false;
					}
					break;
			}
		}

		/// <summary>
		/// Call this method when MouseUp event received
		/// </summary>
		/// <param name="x"></param>
		/// <param name="y"></param>
		public void MouseUp (int x, int y)
		{
			Point point = scale.PointDeviceToWorld(new Point(x,y));
			x=point.X;
			y=point.Y;
			
			switch (state)
			{
				case State.Moving:
					InitialRectangle = MoveSelection(x,y);
					OnFinalPosition();
					state=State.NoSelection;
					break;
			}
		}

		/// <summary>
		/// To cancel current dragging
		/// </summary>
		public void Cancel()
		{
			state=State.NoSelection;
			BuildRectangleIndex(ref initialRectangle, ref area);
			OnFinalPosition();
		}

		/// <summary>
		/// Overridable, change cursor look
		/// </summary>
		public virtual void OnSetCursor(Cursor cursor, int areaIndex)
		{
			if (SetCursorEvent!=null)
				SetCursorEvent(this, cursor, areaIndex);
		}

		/// <summary>
		/// Overridable, user is moving the actual selection
		/// </summary>
		public virtual void OnDragPosition()
		{
			if (DragPositionEvent!=null)
				DragPositionEvent(this);
		}

		/// <summary>
		/// Overridable, user has finish moving
		/// </summary>
		public virtual void OnFinalPosition()
		{
			if (FinalPositionEvent!=null)
				FinalPositionEvent(this);
		}

		/// <summary>
		/// Override this to know when mouse is outside object
		/// Useful to trigger a dragdrop for exemple
		/// </summary>
		/// <param name="bOutside">true if outside, false if inside</param>
		public virtual void OnMouseOutside(bool bOutside, int grip)
		{
			if (MouseOutisedeEvent!=null)
				MouseOutisedeEvent(this, bOutside);
		}

		/// <summary>
		/// Override this to know when element is selected or deselected
		/// </summary>
		/// <param name="bSelected">current selection state</param>
		public virtual void OnSelected(bool bSelected)
		{
			if (SelectionEvent!=null)
				SelectionEvent(this, bSelected);
		}

		private Rectangle RectWorldToDevice (Rectangle rect)
		{
			return new Rectangle(scale.PointWorldToDevice(rect.Location), scale.SizeWorldToDevice(rect.Size));
		}
	}
}
