using System;
using System.Collections.Generic;
using System.Xml.Serialization;
using System.IO;
using System.Windows.Forms;

namespace url2jpeg
{

    public class langItem
    {
        [XmlAttribute]
        public string m_key;
        [XmlAttribute]
        public string m_value;

        public langItem()
        {
        }

        public langItem(string key, string value)
        {
            m_key = key;
            m_value = value;
        }
    }

    public class langCountry
    {
        [XmlAttribute]
        public string                       m_country;
        public List<langItem>               m_items;

        public langCountry()
        {
            m_items = new List<langItem>();
        }

        public langCountry(string country)
        {
            m_items = new List<langItem>();
            m_country = country;
        }

        public string GetItem(string key, string def)
        {
            string result = string.Empty;
            foreach (langItem li in m_items)
            {
                if (li.m_key == key)
                {
                    result = li.m_value;
                    break;
                }
            }
            if (result.Length == 0)
            {
                result = def;
                m_items.Add(new langItem(key, def));
            }
            return result;
        }

        public void Translate(string prefix, List<System.Windows.Forms.Control> controls)
        {
            foreach (System.Windows.Forms.Control ctrl in controls)
            {
                ctrl.Text = GetItem(prefix+"."+ctrl.Name, ctrl.Text);
            }
        }
    }

    public class lang
    {
        public List<langCountry>            m_locale;

        public static lang LoadXML(string fileName)
        {
            lang result;
            try
            {
                XmlSerializer read = new XmlSerializer(typeof(lang));
                using (FileStream input = new FileStream(fileName, FileMode.Open))
                {
                    result = (lang)read.Deserialize(input);
                }
            }
            catch
            {
                // The file is not find, create an new object
                result = new lang();
                result.m_locale = new List<langCountry>();
                result.m_locale.Add(new langCountry("English"));
                result.m_locale.Add(new langCountry("Franais")); // To have unicode file generation
            }
            return result;
        }

        public void SaveXML(string fileName)
        {
            XmlSerializer write = new XmlSerializer(typeof(lang));
            StreamWriter sm = new StreamWriter(fileName);
            write.Serialize(sm, this);
            sm.Close();
        }

        public langCountry FindByCountry(string country)
        {
            langCountry result = null;
            foreach (langCountry lc in m_locale)
            {
                if (country == lc.m_country)
                {
                    result = lc;
                    break;
                }
            }
            return result;
        }
    }

    public class LocalizedUserControl : UserControl
    {
        public virtual void DoControlTranslate(ControlCollection ctrls, langCountry lc)
        {
            List<Control> list = new List<Control>();
            foreach (Control ctrl in ctrls)
            {
                Type t = ctrl.GetType();
                if (t == typeof(Button) || t == typeof(TextBox) || t == typeof(Label) || t==typeof(CheckBox) || t==typeof(RadioButton))
                {
                    if (ctrl.Tag == null && ctrl.Text.Length>0)
                        list.Add(ctrl);
                }
                else
                {
                    DoControlTranslate(ctrl.Controls, lc);
                }
            }
            lc.Translate(Name, list);
        }

        public virtual void DoTranslate(langCountry lc)
        {
            DoControlTranslate(Controls, lc);
        }
    }
}
